import { __spreadArray, __awaiter, __generator } from './../external/tslib/tslib.es6.js';
import React, { useState, useMemo, useEffect, useCallback } from 'react';
import { DndProvider } from 'react-dnd';
import HTML5Backend from 'react-dnd-html5-backend';
import DraggableItem from './DraggableItem.js';
import styles from './DragAndDropList.module.scss.js';

function WithDragBehavior(_a) {
  var _this = this;

  var list = _a.list,
      renderItem = _a.renderItem,
      dragProps = _a.dragProps;

  var _b = useState(list),
      internalList = _b[0],
      setInternalList = _b[1]; // eslint-disable-next-line react-hooks/exhaustive-deps


  var initialList = useMemo(function () {
    return list;
  }, []);
  useEffect(function () {
    setInternalList(list);
  }, [list]);
  useEffect(function () {
    var _a;

    var resetToOriginalList = function () {
      setInternalList(initialList);

      if (dragProps.onReset) {
        dragProps.onReset(initialList);
      }
    };

    if ((_a = dragProps.resetButtonRef) === null || _a === void 0 ? void 0 : _a.current) {
      dragProps.resetButtonRef.current.addEventListener('click', resetToOriginalList);
    }

    return function () {
      var _a;

      if ((_a = dragProps.resetButtonRef) === null || _a === void 0 ? void 0 : _a.current) {
        dragProps.resetButtonRef.current.removeEventListener('click', resetToOriginalList);
      }
    };
  }, [dragProps, initialList]);
  var onMoveItem = useCallback(function (dragIndex, hoverIndex) {
    var newList = __spreadArray([], internalList, true);

    var dragItem = newList[dragIndex];
    newList.splice(dragIndex, 1);
    newList.splice(hoverIndex, 0, dragItem);
    setInternalList(newList);
    return newList;
  }, [internalList]);
  var onDragUndo = useCallback(function (idx, initialIndex) {
    var undoList = onMoveItem(idx, initialIndex);

    if (dragProps.onUndo) {
      dragProps.onUndo(undoList);
    }
  }, [onMoveItem, dragProps]);
  var onDropItem = useCallback(function (item) {
    return __awaiter(_this, void 0, void 0, function () {
      var droppedItem, onSortResult;
      return __generator(this, function (_a) {
        if (dragProps.behavior === 'sort' || dragProps.behavior === 'sortAndDropOutside') {
          droppedItem = internalList.find(function (_a) {
            var key = _a.key;
            return key === item.id;
          });
          if (!droppedItem) return [2
          /*return*/
          ];
          onSortResult = dragProps.onSort(internalList, droppedItem);

          if (onSortResult) {
            onSortResult.catch(function () {
              return onDragUndo(item.index, item.initialIndex);
            });
          }
        }

        return [2
        /*return*/
        ];
      });
    });
  }, [dragProps, internalList, onDragUndo]);
  var onRemoveItem = useCallback(function (item) {
    var newList = __spreadArray([], internalList, true);

    var prevList = __spreadArray([], internalList, true);

    newList.splice(item.index, 1);
    setInternalList(newList);
    return {
      newList: newList,
      prevList: prevList
    };
  }, [internalList]);
  var onDropOutside = useCallback(function (item) {
    if (dragProps.behavior === 'sort' || dragProps.behavior === 'dropInside') {
      return;
    }

    if (!item.dropTarget) return;
    var onDropOutsideResult = dragProps.onDropOutside(internalList[item.index], item.dropTarget);
    var prevList = onRemoveItem(item).prevList;

    if (onDropOutsideResult) {
      onDropOutsideResult.catch(function () {
        setInternalList(prevList);

        if (dragProps.onUndo) {
          dragProps.onUndo(prevList);
        }
      });
    }
  }, [dragProps, internalList, onRemoveItem]);
  return /*#__PURE__*/React.createElement(DndProvider, {
    backend: HTML5Backend
  }, /*#__PURE__*/React.createElement("ul", {
    className: styles.list
  }, internalList.map(function (item, index) {
    return /*#__PURE__*/React.createElement(DraggableItem, {
      validDragComponentIds: dragProps.validDragComponentIds,
      draggableTypeId: dragProps.id,
      key: item.key,
      draggableItem: item,
      index: index,
      onMoveItem: onMoveItem,
      onDropItem: onDropItem,
      onDropOutside: onDropOutside,
      onDragUndo: onDragUndo,
      isDraggable: dragProps.behavior !== 'dropInside',
      isSortable: dragProps.behavior === 'sort' || dragProps.behavior === 'sortAndDropOutside'
    }, function (_a) {
      var dragRef = _a.dragRef,
          canDrop = _a.canDrop,
          isOver = _a.isOver;
      return renderItem({
        item: item,
        index: index,
        dragRef: dragRef,
        isOver: isOver,
        canDrop: canDrop
      });
    });
  })));
}

function DragAndDropList(_a) {
  var list = _a.list,
      renderItem = _a.renderItem,
      dragProps = _a.dragProps,
      listItemClassName = _a.listItemClassName;

  if (!dragProps) {
    return /*#__PURE__*/React.createElement("ul", {
      className: styles.list
    }, list.map(function (item, index) {
      return /*#__PURE__*/React.createElement("li", {
        className: listItemClassName,
        key: item.key
      }, renderItem({
        item: item,
        index: index
      }));
    }));
  }

  return /*#__PURE__*/React.createElement(WithDragBehavior, {
    list: list,
    renderItem: renderItem,
    dragProps: dragProps
  });
}

export { DragAndDropList as default };
//# sourceMappingURL=DragAndDropList.js.map
