import { __spreadArray } from './../external/tslib/tslib.es6.js';
import React, { useRef } from 'react';
import { useDrop, useDrag } from 'react-dnd';
import classnames from 'classnames';
import styles from './DraggableItem.module.scss.js';
var defaultValidDragComponentIDs = [];

function DraggableItem(_a) {
  var _b;

  var children = _a.children,
      isSortable = _a.isSortable,
      isDraggable = _a.isDraggable,
      index = _a.index,
      onMoveItem = _a.onMoveItem,
      onDropItem = _a.onDropItem,
      onDropOutside = _a.onDropOutside,
      onDragUndo = _a.onDragUndo,
      _c = _a.validDragComponentIds,
      validDragComponentIds = _c === void 0 ? defaultValidDragComponentIDs : _c,
      draggableTypeId = _a.draggableTypeId,
      draggableItem = _a.draggableItem;
  var dragIconRef = useRef(null);
  var dropRef = useRef(null);

  var _d = useDrop({
    accept: __spreadArray([draggableTypeId], validDragComponentIds, true),
    collect: function (monitor) {
      return {
        handlerId: monitor.getHandlerId(),
        isOver: monitor.isOver(),
        canDrop: monitor.canDrop()
      };
    },
    canDrop: function (item) {
      var _a;

      return !draggableItem.disabled && !((_a = item.dropTarget) === null || _a === void 0 ? void 0 : _a.active);
    },
    drop: function (item) {
      if (item.type !== draggableTypeId) {
        item.droppableZone = 'outside'; // this cast is needed because the target might be different

        item.dropTarget = draggableItem;
      }

      onDropItem(item);
    },
    hover: function (item, monitor) {
      if (!dragIconRef.current) {
        // If the target area is a different we set the drop target to the one that is being hover over
        item.dropTarget = draggableItem;
        return;
      }

      if (item.type !== draggableTypeId) {
        return;
      }

      var dragIndex = item.index;
      var hoverIndex = index; // Don't replace items with themselves

      if (dragIndex === hoverIndex) {
        return;
      } // Determine rectangle on screen


      var hoverBoundingRect = dragIconRef.current.getBoundingClientRect(); // Get vertical middle

      var hoverMiddleY = (hoverBoundingRect.bottom - hoverBoundingRect.top) / 2; // Determine mouse position

      var clientOffset = monitor.getClientOffset();
      if (!clientOffset) return; // Get pixels to the top

      var hoverClientY = clientOffset.y - hoverBoundingRect.top; // Only perform the move when the mouse has crossed half of the items height
      // When dragging downwards, only move when the cursor is below 50%
      // When dragging upwards, only move when the cursor is above 50%
      // Dragging downwards

      if (dragIndex < hoverIndex && hoverClientY < hoverMiddleY) {
        return;
      } // Dragging upwards


      if (dragIndex > hoverIndex && hoverClientY > hoverMiddleY) {
        return;
      }

      if (isSortable) {
        item.index = hoverIndex;
        onMoveItem(dragIndex, hoverIndex);
      } // Note: we're mutating the monitor item here!
      // Generally it's better to avoid mutations,
      // but it's good here for the sake of performance
      // to avoid expensive index searches.


      item.droppableZone = 'inside';
    }
  }),
      _e = _d[0],
      handlerId = _e.handlerId,
      isOver = _e.isOver,
      canDrop = _e.canDrop,
      drop = _d[1];

  var _f = useDrag({
    item: {
      type: draggableTypeId,
      id: draggableItem.key,
      index: index,
      initialIndex: index,
      droppableZone: 'inside'
    },
    collect: function (monitor) {
      return {
        isDragging: monitor.isDragging()
      };
    },
    canDrag: !draggableItem.disabled && isDraggable,
    end: function (item, monitor) {
      if (!item) {
        return;
      }

      var didDrop = monitor.didDrop();

      if (!didDrop || item.droppableZone === 'outside') {
        onDragUndo(item.index, item.initialIndex);
      }

      if (didDrop && item.droppableZone === 'outside') {
        onDropOutside(item);
      }
    }
  }),
      isDragging = _f[0].isDragging,
      drag = _f[1],
      dragPreview = _f[2];

  dragPreview(dropRef.current, {
    captureDraggingState: true
  });
  drag(dragIconRef);
  drop(dropRef);
  return /*#__PURE__*/React.createElement("li", {
    ref: dropRef,
    "data-handler-id": handlerId,
    className: classnames(styles.draggableItem, (_b = {}, _b[styles.hideItem] = isDragging, _b)),
    key: draggableItem.key
  }, typeof children === 'function' && children({
    dragRef: isDraggable ? dragIconRef : undefined,
    canDrop: canDrop,
    isOver: isOver
  }));
}

export { DraggableItem as default };
//# sourceMappingURL=DraggableItem.js.map
